@@ -46,9 +46,9 @@ image.TOAr <- calcTOAr(image.DN = image.DN, sat="L7", MTL = MTLfile,
 image.SR <- calcSR(image.TOAr=image.TOAr, sat = "L7", 
                    surface.model=surface.model, 
                    incidence.hor = solar.angles.r$incidence.hor, 
-                   WeatherStation=WeatherStation, ESPA = F)
+                   WeatherStation=WeatherStation)
 
-albedo <- albedo(image.SR = image.SR,  coeff="Tasumi")
+albedo <- albedo(image.SR = image.SR,  coeff="Tasumi", sat="L7")
 
 ## ---- fig.width = 5------------------------------------------------------
 LAI <- LAI(method = "metric2010", image = image.TOAr, L=0.1)
@@ -56,7 +56,7 @@ LAI <- LAI(method = "metric2010", image = image.TOAr, L=0.1)
 plot(LAI)
 
 ## ---- warning=FALSE, fig.width = 5---------------------------------------
-Ts <- surfaceTemperature(LAI=LAI, sat = "L7", thermalband = image.DN$thermal.low,
+Ts <- surfaceTemperature(image.DN=image.DN, LAI=LAI, sat = "L7", 
                          WeatherStation = WeatherStation)
 
 Rl.out <- outLWradiation(LAI = LAI, Ts=Ts)
